<!doctype html>
<html lang="en-us" class="layout-pf pf-m-redhat-font">
  <head>
    <meta charset="utf-8">
    <title>Cockpit-Composer</title>

    <!-- js dependencies -->
    <script type="text/javascript" src="../base1/cockpit.js"></script>
    <script type="text/javascript" src="../*/po.js"></script>
  </head>
  <body>
    <div id="main"></div>
  </body>
</html>
