.pf-c-empty-state.pf-m-lg {
  max-width: 100%;
}

/* PF4 has spacing above primary buttons - since we use secondary
 * and also PF3 buttons extend the rule to cover all buttons
 */
.pf-c-empty-state__body + button,
.pf-c-empty-state__body + a > .btn {
  margin-top: var(--pf-global--spacer--xl);
}
