#cmpsr-blueprint-input-filter {
  min-width: 60%;
}

.pf-c-pagination.pf-m-bottom .pf-c-options-menu {
  display: none;
}

.pf-c-pagination.pf-m-bottom {
  --pf-c-pagination--m-bottom--BoxShadow: none;
}

.pf-c-options-menu.pf-m-top .pf-c-options-menu__menu {
  --pf-c-options-menu__menu--Top: var(--pf-c-options-menu__menu--Top);

  transform: translateY(0);
}

/* The base size for smaller viewports in patternfly-cockpit.scss is too small
/* for this context. Using the size defined for larger viewports instead. */
.toolbar-pf-results ul {
  line-height: 40px;
}

@media screen and (min-width: 768px) {
  /* this is when the pagination changes from just previous/next buttons to the full control
  /* at this size and above, 45% is large enough for the input */
  #cmpsr-blueprint-input-filter {
    min-width: 45%;
  }

  .pf-c-pagination.pf-m-bottom {
    padding: 0;
  }

  .pf-c-pagination.pf-m-bottom .pf-c-options-menu {
    display: block;
  }
}
