/*
 * Keep in sync with https://github.com/cockpit-project/cockpit/tree/main/src/base1/patternfly-cockpit.scss
 */

$icon-font-path: 'patternfly-icons-fake-path/';
$font-path: 'patternfly-fonts-fake-path/';

@import "./patternfly-overrides-variables";
@import "~patternfly/dist/sass/patternfly";
@import "./patternfly-4-cockpit.scss";
@import "./patternfly-overrides.scss";
